//! Support mapping based Plane shape.
use crate::math::Vector;
use na::{Real, Unit};

/// SupportMap description of a plane.
#[derive(PartialEq, Debug, Clone)]
#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
pub struct Plane<N: Real> {
    /// The plane normal.
    normal: Unit<Vector<N>>,
}

impl<N: Real> Plane<N> {
    /// Builds a new plane from its center and its normal.
    #[inline]
    pub fn new(normal: Unit<Vector<N>>) -> Plane<N> {
        Plane { normal: normal }
    }

    /// The plane normal.
    #[inline]
    pub fn normal(&self) -> &Unit<Vector<N>> {
        &self.normal
    }
}
